/* *************************************************************************

   I N T R A C H E
   ---------------

   Log analyser & workstation query for intranets running Apache

   Author & Copyright: Vinod L. Hingorani, (c) 2000
   
   Legal: See accompanying file COPYING (GPL)

   AND DON'T SUE ME, THIS IS A FREE SOFTWARE!
   
   The file "COPYING" MUST BE INCLUDED WITH THIS SOURCE CODE AT ALL TIMES.
   
   Date of release: October 25, 2000

   ************************************************************************* */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>


#define MAXLEN 		255
#define LINE80 		80
#define DONE		-111

#define EXIT_NORMAL	0
#define EXIT_MEM_ERROR	1
#define EXIT_FILE_ERROR	2

#define ALLNOVIEW	-1
#define NOVIEW    	0
#define VIEW      	1
#define TXTFILE   	2
#define HTMLFILE  	3

#define DISP_CONTINUE	-11
#define DISP_EXIT	-12
#define DISP_INSIDE	-13
#define DISP_INIT	-14
#define DISP_QUERY	-15



/* structure for workstation data */

typedef struct {
    char nick[10];
    char ip[16];
    char first[14];
    char last[16];
    char phone[10];
    char status[10];
} workstation;


/* structure for page data */

typedef struct {
    char nick[10];
    char pname[200];
} pdata;


/* structure for Apache log file data */

typedef struct {
    char ip[17];
    char adate[33];
    char amethod[10];
    char page[500];
    char version[20];
    char aerror[10];
    char fsize[20];
} logdata;


/* linked list structure for pages */
struct llpg {
    char pname[200];
    struct llpg *next;
};

/* linked list structure for workstations */
struct llws {
    char ip[20];
    struct llws *next;
};

typedef struct llws WS;
typedef WS *WSLINK;

typedef struct llpg PG;
typedef PG *PGLINK;

int asciiz_line(char *s);
int choice(int maxchoice);
int currline_ip_match(int i);
int currline_page_match(int i);
int disp_counter(int max);
int get_dates();
int get_output_filename(char *s);
int get_ws_status(int);
int int_page_id(int);
int int_ws_id(int);
int logdate_to_int(char *log_date) 
;
int ngets(char *prompt, char *s, int ln);
int output_format_menu();
int output_header(int mode, int sitehits, int wsid, int pageid);
int output_log(int mode, int, int);
int prompt_yesno(char *s);
int scan_access_log(int, int, int);
int userdate_to_int(char *user_date) 

;

void add_page();
void add_ws();
void all_pages_ws(void);
void bakup_file(char *first);
void clearscreen(char *menuname);
void currline_parse(char *read_line);
void delete_page();
void delete_ws();
void display_flywheel(int);
void get_all_ws_status();
void help_about();
void load_conf();
void load_page_db();
void load_ws_db();
void maintain_ws();
void prompt_enter();
void quit(int, char *s);
void scan_all_ws();
void show_invalid(char *s, int);
void show_page_details(int i);
void show_ws_details(int);
void sort_file(char *first);
void underline(int);
void view_error_log();
void view_page_db();
void view_ws_db();


/* global variables */

workstation	*Glob_wsdata;	/* array of workstation data structure */
pdata		*Glob_pagedata;	/* array of page data structure */
logdata		Glob_currline; /* single structure instance for current line */

int Glob_maxws;			/* max number of workstations */
int Glob_maxpages;		/* max number of pages */
int Glob_userstart_date;	/* integer value of user-entered start date */
int Glob_userend_date;		/* integer value of user-entered end date */
int *Glob_hitlist;		/* integer array for hits from workstations */

FILE *Glob_outfp;		/* output file stream, kept global */

char Glob_access_log_file[MAXLEN];	/* Apache access log file */
char Glob_error_log_file[MAXLEN];	/* Apache error log file */
char Glob_cmd_ping[MAXLEN];		/* unix/linux ping command */
char Glob_cmd_clear[MAXLEN];		/* clear screen command */
char Glob_cmd_sort[MAXLEN];		/* unix/linux sort command */

char *Glob_ws_dbfile	= "intrache.ws";	/* workstation database */
char *Glob_page_dbfile	= "intrache.page";	/* pages database */
char *Glob_ini_file	= "intrache.conf";	/* configuration file */
char *Glob_help_file	= "intrache.help";	/* help file */


/* -------------------------------------------------------------
   Main - entry point & main menu loop is here
   ------------------------------------------------------------- */
int main()
{
    int wsid, pageid;

    strcpy(Glob_cmd_clear, "clear");

    /*
	Temporarily used till initialised by load_conf() function
        for "clearscreen" function.
        We assume that "clear" will work till then
    */

    clearscreen("Initializing...");

    load_conf();
    load_ws_db();
    load_page_db();

    if (prompt_yesno("Ping workstations for status now?"))
	get_all_ws_status();

    while (1) {
	clearscreen("Main Menu");
	puts("0. Quit\n");
	puts("1. Workstations & pages database maintenance\n");
	puts("2. View webserver error log for [error]\n");
	puts("3. Generate report for workstation");
	puts("4. Generate report for page");
	puts("5. Generate master report\n");
	puts("6. Generate report of hits from all workstations\n");
	puts("7. Help & About Intrache\n");

	switch (choice(7)) {
	case 0:
	    {
		if (prompt_yesno("Quit: Are you sure?"))
		    quit(EXIT_NORMAL, "");
		break;
	    }
	case 1:
	    maintain_ws();
	    break;
	case 2:
	    view_error_log();
	    break;
	case 3:
	    {
		clearscreen("Report by Workstation");
		if ((wsid = int_ws_id(0)) == 0)
		    break;
		pageid = int_page_id(1);
		if (get_dates() == 0)
		    scan_access_log(NOVIEW, wsid, pageid);
		break;
	    }
	case 4:
	    {
		clearscreen("Report by Page visited");
		if ((pageid = int_page_id(0)) == 0)
		    break;
		wsid = int_ws_id(1);
		if (get_dates() == 0)
		    scan_access_log(NOVIEW, wsid, pageid);
		break;
	    }
	case 5:
	    {
		clearscreen ("Report for all known workstations & pages");
		if (get_dates() == 0)
		    scan_access_log(NOVIEW, 0, 0);
		break;
	    }
	case 6:
	    scan_all_ws();
	    break;
	case 7:
	    help_about();
	    break;
	}
    }
}

/* -------------------------------------------------------------
    Choice: Used by all menu input functions. Used "scanf" for input,
    It's ANSI, but not too good - it's buffered!.
    returns integer of choice from 0 to maxchoice
   ------------------------------------------------------------- */
int choice(int maxchoice)
{
    int i = 0;
    char c[MAXLEN];
    do {
	underline(LINE80);
	printf("Enter selection (0-%d): ", maxchoice);
	scanf("%s", c);
	i = atoi(c);
    } while (i < 0 || i > maxchoice);
    return (i);
}


/* -------------------------------------------------------------
    Get user input string with a prompt. returns 0 if input exceeds 
    max length of characters, else returns strlen of input.
    Again uses "scanf" <sigh!>
   ------------------------------------------------------------- */
int ngets(char *prompt, char *s, int max)
{
    char t[MAXLEN];
    printf("Enter %s: ", prompt);
    scanf("%s", t);

    if (strlen(t) > max)
	return (0);
    strcpy(s, t);
    return (strlen(s));
}


/* -------------------------------------------------------------
    Used to replace '\n' by 0 for making ASCIIZ string from line read
   ------------------------------------------------------------- */
int asciiz_line(char *s)
{
    int i;
    for (i = 0; i < strlen(s); i++) {
	if (s[i] == '\n') {
	    s[i] = 0;
	    return (i);
	}
    }
}

/* -------------------------------------------------------------
    Underline on the screen with "-"
   ------------------------------------------------------------- */
void underline(int max)
{
    int i;
    for (i = 0; i < max; i++)
	putchar('-');
    putchar('\n');
}


/* -------------------------------------------------------------
    Prompt with <enter> to continue
    One of the pitfalls of using ANSI functions for keyboard IO
    All input functions are buffered, so we have to clear the buffers
    by dummy reads and flushes.
   ------------------------------------------------------------- */
void prompt_enter()
{
    char ch = 0;
    char t[MAXLEN];
    underline(LINE80);
    printf("Press <Enter> to continue");
    fgets(t, MAXLEN, stdin);		/* flush keyboard */
    while (1) {
	fflush(stdin);			/* pull the chain again */
	ch = getchar();
	if (ch == '\n')
	    return;
     }
}

/* -------------------------------------------------------------
    Prompt with Yes or No to continue. Returns 1 for "Y", 0 for "N"
   ------------------------------------------------------------- */
int prompt_yesno(char *s)
{
    char ch;
    underline(LINE80);
    printf("%s (Enter Y or N) ", s);
    fflush(stdin);			/* flush keyboard */
    while (1) {
	fflush(stdin);			/* pull the chain again */
	ch = getchar();
	if ((ch == 'n') || (ch == 'N'))
	    return (0);
	if ((ch == 'y') || (ch == 'Y'))
	    return (1);
    }
}


/* -------------------------------------------------------------
    Used for counting lines of displayed output, so user can pause
    for every max lines displayed.
    max = DISP_INIT is used to intilialize, counter = 0
    max = DISP_QUERY is used to return current counter value
    returns DISP_INSIDE if within max lines, NO PROMPT IS GIVEN, counter++
    returns DISP_EXIT if user presses "N" for NO, counter = 0
    returns DISP_CONTINUE if user presses "Y" for YES, counter = 0
   ------------------------------------------------------------- */
int disp_counter(int max)
{
    static int count;
    
    if (max == DISP_INIT) {
	count = 0;
	return(count);
    }
	
    if (max == DISP_QUERY)
	return(count);

    if ((++count) < max)
	return(DISP_INSIDE);

    count = 0;

    if (prompt_yesno("Display more?"))
	return(DISP_CONTINUE);
    else
	return(DISP_EXIT);
}

/* -------------------------------------------------------------
    Quits from program. What else?
    This is used as a single exit point, that's all!
   ------------------------------------------------------------- */
void quit(int i, char *s)
{
    if (i == EXIT_NORMAL) {
	clearscreen("Thanks for using Intrache");
	free(Glob_wsdata);
	free(Glob_pagedata);
	free(Glob_hitlist);
	exit(0);
    }

    if (i == EXIT_FILE_ERROR) {
printf("\n\nCannot find or open file: %s\n\nAborting program ...\n\n", s);
	exit(EXIT_FILE_ERROR);
    }

    if (i == EXIT_MEM_ERROR) {
printf("\n\nCannot allocate memory for %s\n\nAborting program ...\n\n", s);
	exit(EXIT_MEM_ERROR);
    }
}


/* -------------------------------------------------------------
    Help & About info
   ------------------------------------------------------------- */
void help_about()
{
    FILE *fp;
    char line[MAXLEN];
    disp_counter(DISP_INIT);
    
    if ((fp = fopen(Glob_help_file, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_help_file);

    while (fgets(line, MAXLEN, fp) != NULL) {
	if(disp_counter(DISP_QUERY) == 0)
	    clearscreen(\
"Intrache (c) 2000 Vinod L. Hingorani.       Email: vinodlh@yahoo.com");
	asciiz_line(line);
	puts(line);
	if (disp_counter(16) == DISP_EXIT) {
	    fclose(fp);
	    return;
	}
    }
    fclose(fp);
    prompt_enter();
}


/* -------------------------------------------------------------
    Clearscreen - by using the system's clear screen command. And
    then print the copyright on top and a heading below it.
   ------------------------------------------------------------- */
void clearscreen(char *menu1)
{
    system(Glob_cmd_clear);
puts("Intrache - Log analyser for Apache.\t\t(c) 2000, Vinod L. Hingorani");
    underline(LINE80);
    puts("");
    puts(menu1);
    if (strlen(menu1))
	underline(strlen(menu1));
    puts("");
}

/* -------------------------------------------------------------
    Get start & end dates for log analysis from user, and then convert
    dates to integer using "userdate_to_int" function.
    The start & end dates are stored as GLOBAL integers
   ------------------------------------------------------------- */
int get_dates()
{
    char start_date[MAXLEN], end_date[MAXLEN];
    char *t = "Please check & re-enter dates in the format specified";


    if (ngets("start date [dd/mm/yyyy]....", start_date, 12) == 0)
	return (-1);

    if ((Glob_userstart_date = userdate_to_int(start_date)) == -1) {
	puts(t);
	prompt_enter();
	return (-1);
    }
    if (ngets("end date [dd/mm/yyyy]......", end_date, 12) == 0)
	return (-1);

    if ((Glob_userend_date = userdate_to_int(end_date)) == -1) {
	puts(t);
	prompt_enter();
	return (-1);
    }
    return (0);
}


/*---------------------------------------------------------------
    Converts a user date string to an integer.
    dd/mm/yyyy is converted to integer of the value yyyymmdd
    useful for checking dates.
----------------------------------------------------------------*/

int userdate_to_int(char *userdate)
{
    char day[4], month[5], year[6], date_rev[16];

    if (strlen(userdate) != 10)
	return (-1);

    if ((userdate[2] != '/') || (userdate[5] != '/'))
	return (-1);

    date_rev[0] = '\0';
    strcpy(day, strtok(userdate, "/"));		/* interchange day & month */
    strcpy(month, strtok(NULL, "/"));		/* for American style date */
    strcpy(year, strtok(NULL, ""));

    if ((atoi(year) <= 1990) || (atoi(year) >= 2050))
	return (-1);

    if ((atoi(month) < 1) || (atoi(month) > 12))
	return (-1);

    if ((atoi(day) < 1) || (atoi(day) > 31))
	return (-1);

    strcat(date_rev, year);
    strcat(date_rev, month);
    strcat(date_rev, day);
    return (atoi(date_rev));
}


/* -------------------------------------------------------------
    Converts a date string from APACHE log file to an integer.
    Useful for checking date against another.
    Output integer is value of "yyyymmdd"
    Input Apache date is of type: "[10/Oct/2000:12:53:47 +0530]"
   ------------------------------------------------------------- */
int logdate_to_int(char *log_orig_date)
{
    char log_date[40], day[4], monthname[5], year[6], log_rev[10];
    int i;
    char months[12][4] =
    {"Jan", "Feb", "Mar", "Apr", "May", "Jun", \
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
    char monthnums[12][3] =
    {"01", "02", "03", "04", "05", "06", \
     "07", "08", "09", "10", "11", "12"};

    strcpy(log_date, log_orig_date);
    log_rev[0] = '\0';

    strcpy(day, strtok(log_date, "/"));
    strcpy(monthname, strtok(NULL, "/"));
    strcpy(year, strtok(NULL, ":"));

    for (i = 0; i < 12; i++) {
	if (strcmp(months[i], monthname) == 0)
	    break;
    }
    strcat(log_rev, year);
    strcat(log_rev, monthnums[i]);
    strcat(log_rev, day);
    return (atoi(log_rev));
}

/* -------------------------------------------------------------
    Maintenance sub-menu for workstations & pages.
    Also options for PINGING workstations for connectivity
   ------------------------------------------------------------- */
void maintain_ws()
{
    int wsid = 0;
    while (1) {
	clearscreen("Database maintenance for workstations & pages");
	puts("0. Exit to Main Menu\n");
	puts("1. View Workstation database");
	puts("2. Add to Workstation database (#9 to see what to add)");
	puts("3. Delete Workstation from database\n");
	puts("4. Ping ALL workstations for status");
	puts("5. Ping SINGLE workstation for status\n");
	puts("6. View page database");
	puts("7. Add to page database (#9 to see what to add)");
	puts("8. Delete page from database\n");
	puts("9. Scan logfile to see workstations & pages");
	puts("   (To see what to add to workstation & pages database)\n");

	switch (choice(9)) {
	case 0:
	    return;
	case 1:
	    view_ws_db();
	    break;
	case 2:
	    add_ws();
	    break;
	case 3:
	    delete_ws();
	    break;
	case 4:
	    get_all_ws_status();
	    break;
	case 5:
	    {
		clearscreen("Ping workstation NOW & view status");
		if (wsid = int_ws_id(0)) {
		    puts("Please wait ...");
		    get_ws_status(wsid);
		    show_ws_details(wsid);
		    prompt_enter();
		}
		break;
	    }
	case 6:
	    view_page_db();
	    break;
	case 7:
	    add_page();
	    break;
	case 8:
	    delete_page();
	    break;
	case 9:
	    all_pages_ws();
	    break;
	}
    }
}


/* -------------------------------------------------------------
    This one displays table for showing all workstations that are
    in the database. Also shows connectivity
   ------------------------------------------------------------- */
void view_ws_db()
{
    int j;
    disp_counter(DISP_INIT);		/* initialize the display counter */
    
    for (j = 1; j <= Glob_maxws; j++) {
	if (disp_counter(DISP_QUERY) == 0)	/* query it */
	{
	    clearscreen("Workstation status table");
	    puts("Nickname  IP address       Connect?    Owner");
	    underline(LINE80);
	}

	printf("%-10s%-17s", Glob_wsdata[j].nick, Glob_wsdata[j].ip);
	printf("%-12s", Glob_wsdata[j].status);
	printf("%s %s\n", Glob_wsdata[j].first, Glob_wsdata[j].last);	
	if (j == Glob_maxws)
	    break;
	if (disp_counter(12) == DISP_EXIT)
	    return;
    }
    prompt_enter();
}


/* -------------------------------------------------------------
    This one displays table for showing all pages that are
    in the database.
   ------------------------------------------------------------- */
void view_page_db()
{
    int j;
    disp_counter(DISP_INIT);

    for (j = 1; j <= Glob_maxpages; j++) {
	if(disp_counter(DISP_QUERY) == 0) {
	    clearscreen("Page database");
	    puts("Nickname    Page name");
	    underline(LINE80);
	}
	printf("%-12s%s\n", Glob_pagedata[j].nick, Glob_pagedata[j].pname);
	if (j == Glob_maxpages)
	    break;
	if (disp_counter(12) == DISP_EXIT)
	    return;
    }
    prompt_enter();
}


/* -------------------------------------------------------------
    Display Apache error log file. Displays lines only if it contains
    the [error] sub-string. [notice] & [warn] are ignored.
   ------------------------------------------------------------- */
void view_error_log()
{
    FILE *fp;
    char line[1024];
    disp_counter(DISP_INIT);

    if ((fp = fopen(Glob_error_log_file, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_error_log_file);

    while (fgets(line, 1024, fp) != NULL) {
	if(disp_counter(DISP_QUERY) == 0)
	    clearscreen("View error log");
	if (asciiz_line(line) == 0)
	    continue;
	if (strstr(line, "[error]") == NULL)
	    continue;
	puts(line);
	if (disp_counter(6) == DISP_EXIT) {
	    fclose(fp);
	    return;
	}
    }
    prompt_enter();
    fclose(fp);
}


/* -------------------------------------------------------------
    Add new workstation details to the workstation database file.
    Sorts the file after adding details. creates a bakup file as well
   ------------------------------------------------------------- */
void add_ws()
{
    workstation tmp;
    int i;

    clearscreen("Add Workstation details");
    puts("Do not enter space, comma or tab\n");

    while(1) {
	if (ngets("nickname.....", tmp.nick, 10) == 0)
	    return;
	for (i = 1; i <= Glob_maxws; i++) {
	    if (strcmp(Glob_wsdata[i].nick, tmp.nick) == 0) {
		puts("This nickname exists in database! Enter another.\n");
		break;
	    }
	}
	if (i > Glob_maxws)
	    break;
    }

    while(1) {
	if (ngets("IP address...", tmp.ip, 15) == 0)
	    return;
	for (i = 1; i <= Glob_maxws; i++) {
	    if (strcmp(Glob_wsdata[i].ip, tmp.ip) == 0) {
		puts("This IP exists in database! Enter another.\n");
		break;
	    }
	}
	if (i > Glob_maxws)
	    break;
    }

    if (ngets("first name...", tmp.first, 20) == 0)
	return;
    if (ngets("last name....", tmp.last, 20) == 0)
	return;
    if (ngets("phone........", tmp.phone, 15) == 0)
	return;
    if (prompt_yesno("Add the above Workstation details now?") == 0)
	return;

    if (get_output_filename(Glob_ws_dbfile) == -1)
	return;

    fprintf(Glob_outfp, "%s \t", tmp.nick);
    fprintf(Glob_outfp, "%s \t", tmp.ip);
    fprintf(Glob_outfp, "%s \t", tmp.first);
    fprintf(Glob_outfp, "%s \t", tmp.last);
    fprintf(Glob_outfp, "%s\n", tmp.phone);
    fflush(Glob_outfp);
    fclose(Glob_outfp);
    free(Glob_wsdata);
    sort_file(Glob_ws_dbfile);
    load_ws_db();
    prompt_enter();
}


/* -------------------------------------------------------------
    Add new page details to the page database file.
    Sorts the file after adding details. creates a bakup file as well
   ------------------------------------------------------------- */
void add_page()
{
    pdata tmp;
    int i;

    clearscreen("Add page details");
    puts("Do not enter space or , or tab\n");


    while(1) {
	if (ngets("nickname.....", tmp.nick, 10) == 0)
	    return;
	for (i = 1; i <= Glob_maxws; i++) {
	    if (strcmp(Glob_pagedata[i].nick, tmp.nick) == 0) {
		puts("This nickname exists in database! Enter another.\n");
		break;
	    }
	}
	if (i > Glob_maxpages)
	    break;
    }

    if (ngets("exact page name....", tmp.pname, 200) == 0)
	return;

    if (prompt_yesno("Add the above page details now?") == 0)
	return;

    if (get_output_filename(Glob_page_dbfile) == -1)
	return;

    fprintf(Glob_outfp, "%s\t", tmp.nick);
    fprintf(Glob_outfp, "%s\n", tmp.pname);
    fflush(Glob_outfp);
    fclose(Glob_outfp);
    free(Glob_pagedata);
    sort_file(Glob_page_dbfile);
    load_page_db();
    prompt_enter();
}


/* -------------------------------------------------------------
    Deletes selected workstation details from the workstation
    database file. Creates bakup first.
   ------------------------------------------------------------- */
void delete_ws()
{
    int i, wsid;

    clearscreen("Delete Workstation from database");
    if ((wsid = int_ws_id(0)) == 0)
	return;
    show_ws_details(wsid);
    if (prompt_yesno("Delete the above Workstation entry?") == 0)
	return;

    bakup_file(Glob_ws_dbfile);

    if (get_output_filename(Glob_ws_dbfile) == -1)
	return;
    for (i = 1; i <= Glob_maxws; i++) {
	if (i != wsid) {
	    fprintf(Glob_outfp, "%s\t", Glob_wsdata[i].nick);
	    fprintf(Glob_outfp, "%s\t", Glob_wsdata[i].ip);
	    fprintf(Glob_outfp, "%s\t", Glob_wsdata[i].first);
	    fprintf(Glob_outfp, "%s\t", Glob_wsdata[i].last);
	    fprintf(Glob_outfp, "%s\n", Glob_wsdata[i].phone);
	}
    }
    fflush(Glob_outfp);
    fclose(Glob_outfp);
    free(Glob_wsdata);
    free(Glob_hitlist);
    load_ws_db();
}


/* -------------------------------------------------------------
    Deletes selected page details from the page database file.
    Creates bakup first.
   ------------------------------------------------------------- */
void delete_page()
{
    int i, pageid;

    clearscreen("Delete page from database");
    if ((pageid = int_page_id(0)) == 0)
	return;
	
    show_page_details(pageid);

    if (prompt_yesno("Delete the above page entry?") == 0)
        return;
    bakup_file(Glob_page_dbfile);

    if (get_output_filename(Glob_page_dbfile) == -1)
        return;
    for (i = 1; i <= Glob_maxpages; i++) {
        if (i != pageid) {
	    fprintf(Glob_outfp, "%s\t", Glob_pagedata[i].nick);
	    fprintf(Glob_outfp, "%s\n", Glob_pagedata[i].pname);
	}
    }
    fflush(Glob_outfp);
    fclose(Glob_outfp);
    free(Glob_pagedata);
    load_page_db();
}


/* -------------------------------------------------------------
    Sorts a text file using the linux (unix) command "sort".
    First, calls "bakup_file" to create a bakup.
   ------------------------------------------------------------- */
void sort_file(char *filename)
{
    char tmp[MAXLEN];

    bakup_file(filename);	/* filename becomes filename~ */
    
    strcpy(tmp, Glob_cmd_sort);	/* "sort -o"			*/
    strcat(tmp, filename);	/* "sort -o filename"		*/
    strcat(tmp, " ");		/* "sort -o filename "  	*/
    strcat(tmp, filename);	/* "sort -o filename filename"	*/
    strcat(tmp, ".~");		/* "sort -o filename filename~"	*/
    system(tmp);		/* EXECUTE THIS COMMAND! 	*/
}


/* -------------------------------------------------------------
    Creates a bakup file using system calls
   ------------------------------------------------------------- */
void bakup_file(char *filename)
{
    char tmp[MAXLEN];
    strcpy(tmp, filename);
    strcat(tmp, "~");		/* tmp = filename~ */
    remove(tmp);		/* delete the filename~ if it exists! */
    rename(filename, tmp);	/* rename the filename as filename~ */
}


/*  -------------------------------------------------------------
    Well, this routine does this:
    Scan the log file, check names of pages & workstation IPs.

    Basically create linked lists for these 2 and display them.
    So that the user can see what to add in the pages and
    workstation database.
    ------------------------------------------------------------- */
void all_pages_ws(void)
{
    int j = 0, i, pcount = 0, wscount = 0;
    char line[1024];
    FILE *fp;

    PGLINK pgnew = NULL;
    PGLINK pghead = NULL;
    PGLINK pgcurrent = NULL;

    WSLINK wsnew = NULL;
    WSLINK wshead = NULL;
    WSLINK wscurrent = NULL;

    /* initialize head for workstation linked list */
        
    if ((wsnew  = (WSLINK) malloc(sizeof(WS))) == NULL)
	quit(EXIT_MEM_ERROR, "wshead");
    wsnew->next = wshead;
    wshead = wsnew;

    /* initialize head for pages linked list */

    if ((pgnew  = (PGLINK) malloc(sizeof(PG))) == NULL)
	quit(EXIT_MEM_ERROR, "pghead");
    pgnew->next = pghead;
    pghead = pgnew;

    clearscreen("List of all workstation IPs and pages in log file");

    if ((fp = fopen(Glob_access_log_file, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_access_log_file);

    printf("\nScanning for all workstations & pages in log file .... ");

    while ((fgets(line, 1024, fp)) != NULL) {
	if (asciiz_line(line) == 0)
	    continue;
    	currline_parse(line);
	display_flywheel(j);
	
	/* scan if the current line's ip matches any in stored list */
	i = 1;
	wscurrent = wshead;
	while(wscurrent->next != NULL) {
	    if (strcmp(Glob_currline.ip, wscurrent->ip) == 0) {
		i = 0;			/* yes it matches, so get out */
		break;
	    }
	    wscurrent = wscurrent->next;
	}

	/* if it doesn't match, then create a new node (link) for it */

	if (i) {
	    j++;
	    wscount++;
	    wscurrent = wshead;
	    while(wscurrent->next != NULL)
		wscurrent = wscurrent->next;
	    if ((wsnew  = (WSLINK) malloc(sizeof(WS))) == NULL)
		quit(EXIT_MEM_ERROR, "wsnew");
	    wscurrent->next = wsnew;
	    wsnew->next = NULL;	    
	    strcpy (wscurrent->ip, Glob_currline.ip);
	}


	/* scan if the current line's page matches any in stored list */

	i = 1;
	pgcurrent = pghead;
	while(pgcurrent->next != NULL) {
	    if (strcmp(Glob_currline.page, pgcurrent->pname) == 0) {
		i = 0;			/* yes it matches, so get out */
		break;
	    }
	    pgcurrent = pgcurrent->next;
	}

	/* if it doesn't match, then create a new node (link) for it */

	if ((i) && ((strstr(Glob_currline.page, ".htm")) ||
		(strcmp(Glob_currline.page, "/") == 0))) {
	    j++;
	    pcount++;
	    pgcurrent = pghead;
	    while(pgcurrent->next != NULL)
		pgcurrent = pgcurrent->next;
	    if ((pgnew  = (PGLINK) malloc (sizeof(PG))) == NULL)
		quit(EXIT_MEM_ERROR, "pgnew");
	    pgcurrent->next = pgnew;
	    pgnew->next = NULL;	    
	    strcpy (pgcurrent->pname, Glob_currline.page);
	}
    }

    display_flywheel(DONE);
    fclose(fp);

    printf("\n\nFound %d unique workstation IPs", wscount);
    printf("\n\nFound %d unique pages (htm & html)\n\n", pcount);

    if (prompt_yesno("Display workstation IPs")) {
	disp_counter(DISP_INIT);    
	wscurrent = wshead;
	while(wscurrent->next != NULL) {
	    if (disp_counter(DISP_QUERY) == 0)
		clearscreen("List of all accessing IPs in log file");
	    puts(wscurrent->ip);
	    wscurrent = wscurrent->next;
	    if (disp_counter(16) == DISP_EXIT)
		break;
	}
    }

    if (prompt_yesno("Display pages")) {
	disp_counter(DISP_INIT);    
	pgcurrent = pghead;
	while(pgcurrent->next != NULL) {
	    if (disp_counter(DISP_QUERY) == 0)
		clearscreen("List of all accessed pages in log file");
	    puts(pgcurrent->pname);
	    pgcurrent = pgcurrent->next;
	    if (disp_counter(16) == DISP_EXIT)
		break;
	}
    }
    prompt_enter();

    /* now that we're done, free the memory */
    
    wscurrent = wshead;
    while(wscurrent != NULL) {
	wsnew = wscurrent->next;
	free(wscurrent);
	wscurrent = wsnew;
    }
    pgcurrent = pghead;
    while(pgcurrent != NULL) {
	pgnew = pgcurrent->next;
	free(pgcurrent);
	pgcurrent = pgnew;
    }
}

/* -------------------------------------------------------------
   Get a string from user for Workstation nickname & compare it
   with structure array to determine which array index it is.
   This is the most important user interaction.
   If the int "all" is true, then prompts are different
   ------------------------------------------------------------- */
int int_ws_id(int all)
{
    int i;
    char tmp[MAXLEN];
    char s[10];

    strcpy(tmp, "nickname for workstation (or 0 ");

    if (all)
	strcat(tmp, "for all workstations)");
    else
	strcat(tmp, "to exit)");

    if (ngets(tmp, s, 10) == 0)
	return (0);

    if ((strlen(s) == 1) && (s[0] == '0'))
	return (0);

    for (i = 1; i <= Glob_maxws; i++) {
	if (strcmp(s, Glob_wsdata[i].nick) == 0)
	    return (i);
    }
    show_invalid ("workstation", all);
    return (0);
}

/* -------------------------------------------------------------
   Get a string from user for page nickname & compare it
   with structure array to determine which array index it is.
   This is the most important user interaction.
   If the int "all" is true, then prompts are different
   ------------------------------------------------------------- */
int int_page_id(int all)
{
    int i;
    char tmp[MAXLEN];
    char s[10];

    strcpy(tmp, "nickname for page (or 0 ");

    if (all)
	strcat(tmp, "for all pages)");
    else
	strcat(tmp, "to exit)");

    if (ngets(tmp, s, 10) == 0)
	return (0);

    if ((strlen(s) == 1) && (s[0] == '0'))
	return (0);

    for (i = 1; i <= Glob_maxpages; i++) {
	if (strcmp(s, Glob_pagedata[i].nick) == 0)
	    return (i);
    }
    show_invalid ("page", all);
    return (0);
}


/* -------------------------------------------------------------
    Used by int_ws_id & int_page_id functions for prompt
   ------------------------------------------------------------- */
void show_invalid(char *s, int i)
{
    printf("\nInvalid %s nickname. ", s);
    if (i == 0)
	printf("Check from view database table in main menu 1\n");
    else
	printf("Using 0 for all %ss\n", s);
    prompt_enter();
}


/* -------------------------------------------------------------
    Show page details for index i.
   ------------------------------------------------------------- */
void show_page_details(int i)
{
    printf("\tNickname.......: %s\n", Glob_pagedata[i].nick);
    printf("\tPage details...: %s\n", Glob_pagedata[i].pname);
}


/* -------------------------------------------------------------
    Show workstation details for index i.
   ------------------------------------------------------------- */
void show_ws_details(int i)
{
    printf("\tNickname.......: %s\n", Glob_wsdata[i].nick);
    printf("\tIP.............: %s\n", Glob_wsdata[i].ip);
    printf("\tFirst name.....: %s\n", Glob_wsdata[i].first);
    printf("\tLast name......: %s\n", Glob_wsdata[i].last);
    printf("\tPhone .........: %s\n", Glob_wsdata[i].phone);
    printf("\tStatus.........: %s\n\n", Glob_wsdata[i].status);
}


/* -------------------------------------------------------------
    Ping a specific workstation for it's status
   ------------------------------------------------------------- */
int get_ws_status(int wsid)
{
    char pingstring[MAXLEN];

    pingstring[0] = '\0';
    strcpy(pingstring, Glob_cmd_ping);
    strcat(pingstring, " ");

    strcat(pingstring, Glob_wsdata[wsid].ip);
    strcat(pingstring, " > /dev/null");

    if (system(pingstring)) {
	strcpy(Glob_wsdata[wsid].status, "No *");
	return (-1);
    }
    else {
	strcpy(Glob_wsdata[wsid].status, "Ok");
	return (0);
    }
}


/* -------------------------------------------------------------
    Ping all workstations for status by calling "get_ws_status"
    in a for loop.
   ------------------------------------------------------------- */
void get_all_ws_status()
{
    int i;
    printf("\nPinging workstations for status data now! Please wait .... ");

    for (i = 1; i <= Glob_maxws; i++)
    {
	display_flywheel(i);
	fflush(stdout);
	fflush(stdin);
	get_ws_status(i);
    }
    display_flywheel(DONE);
}



/* -------------------------------------------------------------
   This function reads content from the configuration file
   ------------------------------------------------------------- */
void load_conf()
{
    FILE *fp;
    char line[MAXLEN];
    int i;

    puts("Reading configuration file:");
    if ((fp = fopen(Glob_ini_file, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_ini_file);

    i = 0;
    while (fgets(line, MAXLEN, fp) != NULL) {
	if ((asciiz_line(line)) && (line[0] != '#')) {
	    i++;
	    switch (i) {
	    case 1:
		strcpy(Glob_access_log_file, line);
		break;
	    case 2:
		strcpy(Glob_error_log_file, line);
		break;
	    case 3:
		strcpy(Glob_cmd_ping, line);
		break;
	    case 4:
		strcpy(Glob_cmd_clear, line);
		break;
	    case 5:
		strcpy(Glob_cmd_sort, line);
		break;
	    }
	}
    }
    printf("\tMain log file....: %s\n", Glob_access_log_file);
    printf("\tError log file...: %s\n", Glob_error_log_file);
    printf("\tPing command.....: %s\n", Glob_cmd_ping);
    printf("\tClear command....: %s\n", Glob_cmd_clear);
    printf("\tSort command.....: %s\n\n", Glob_cmd_sort);
    fclose(fp);
}


/* -------------------------------------------------------------
   This function reads content from the workstation database file
   and stores in the structure Glob_wsdata
   ------------------------------------------------------------- */
void load_ws_db()
{
    FILE *fp;
    char line[MAXLEN];
    int i;
    char *mask = " \t\n";

    printf("Reading Workstation database file .... ");
    if ((fp = fopen(Glob_ws_dbfile, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_ws_dbfile);

    i = 1;
    while (fgets(line, MAXLEN, fp) != NULL) {
	display_flywheel(i);
	if ((asciiz_line(line)) && (line[0] != '#'))
	    i++;
    }
    rewind(fp);

    Glob_wsdata = (workstation *) malloc((i + 2) * sizeof(workstation));
    if (Glob_wsdata == NULL)
	quit(EXIT_MEM_ERROR, "wsdata");

    Glob_hitlist = (int *) malloc((i + 2) * sizeof(int));
    if (Glob_hitlist == NULL)
	quit(EXIT_MEM_ERROR, "hitlist");

    i = 1;
    while (fgets(line, MAXLEN, fp) != NULL) {
	display_flywheel(i);
	if ((asciiz_line(line)) && (line[0] != '#')) {
	    strcpy(Glob_wsdata[i].nick, strtok(line, mask));
	    strcpy(Glob_wsdata[i].ip, strtok(NULL, mask));
	    strcpy(Glob_wsdata[i].first, strtok(NULL, mask));
	    strcpy(Glob_wsdata[i].last, strtok(NULL, mask));
	    strcpy(Glob_wsdata[i].phone, strtok(NULL, mask));

	    strcpy(Glob_wsdata[i].status, "NoScan");

	    Glob_maxws = i;
	    i++;
	}
    }
    fclose(fp);
    display_flywheel(DONE);
    printf("\tTotal workstations listed: %d\n\n", Glob_maxws);
}

/* -------------------------------------------------------------
   This function reads content from the page database file and
   stores in the structure Glob_pagedata
   ------------------------------------------------------------- */
void load_page_db()
{
    FILE *fp;
    char line[MAXLEN];
    int i;

    printf("Reading page database file .... ");
    if ((fp = fopen(Glob_page_dbfile, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_page_dbfile);

    i = 1;
    while (fgets(line, MAXLEN, fp) != NULL) {
	display_flywheel(i);
	if ((asciiz_line(line)) && (line[0] != '#'))
	    i++;
    }
    rewind(fp);
    Glob_pagedata = (pdata *) malloc((i + 2) * sizeof(pdata));

    if (Glob_pagedata == NULL)
	quit(EXIT_MEM_ERROR, "pagedata");

    i = 1;
    while (fgets(line, MAXLEN, fp) != NULL) {
	display_flywheel(i);
	if ((asciiz_line(line)) && (line[0] != '#')) {
	    strcpy(Glob_pagedata[i].nick, strtok(line, ", \t"));
	    strcpy(Glob_pagedata[i].pname, strtok(NULL, ", \t\n"));
	    Glob_maxpages = i;
	    i++;
	}
    }
    fclose(fp);
    display_flywheel(DONE);
    printf("\tTotal pages listed: %d\n\n", Glob_maxpages);
}


/* -------------------------------------------------------------
    The "What format to output scan data?" menu
   ------------------------------------------------------------- */
int output_format_menu()
{
    underline(LINE80);
    puts \
("Output mode:   0-Exit   1-View   2-Save as text file   3-Save as HTML file");
    return (choice(3));
}


/* -------------------------------------------------------------
    Scan for all known workstations, known pages and output
    the resultant data as required.
   ------------------------------------------------------------- */
void scan_all_ws()
{
    int i, pageid, totalhits;

    clearscreen("Total number of hits for all workstations");
    pageid = int_page_id(1);

    if (get_dates() == -1)
	return;

    totalhits = scan_access_log(ALLNOVIEW, 0, pageid);

    while (1) {
	switch (output_format_menu()) {
	case NOVIEW:
	    return;
	case VIEW:
	    {
		disp_counter(DISP_INIT);
		output_header(VIEW, totalhits, 0, pageid);
		for (i = 1; i <= Glob_maxws; i++) {
		    if(disp_counter(DISP_QUERY) == 0) {
			puts("Nickname  IP address            Hits   Owner");
			underline(LINE80);
		    }
		    printf("%-10s%-17s", Glob_wsdata[i].nick, \
			    Glob_wsdata[i].ip);
		    printf("%9d   ", Glob_hitlist[i]);
    		    printf("%s %s\n", Glob_wsdata[i].first, \
			    Glob_wsdata[i].last);

		    if (i == Glob_maxws)
			break;
		    if (disp_counter(10) == DISP_EXIT)
			break;
		}
		break;
	    }
	case TXTFILE:
	    {
		if ((output_header(TXTFILE, totalhits, 0, pageid)) != -1) {
		    fprintf(Glob_outfp,\
			    "Nickname\tIP\tHits\tOwner\n\n");
		    for (i = 1; i <= Glob_maxws; i++) {
			fprintf(Glob_outfp, "%s\t%s\t%d\t%s %s\n", \
				Glob_wsdata[i].nick, \
			   Glob_wsdata[i].ip, Glob_hitlist[i], \
			   Glob_wsdata[i].first, Glob_wsdata[i].last);
		    }
		    display_flywheel(DONE);
		    fclose(Glob_outfp);
		}
		break;
	    }
	case HTMLFILE:
	    {
		if ((output_header(HTMLFILE, totalhits, 0, pageid)) != -1) {
		    fprintf(Glob_outfp, \
			    "<tr><td>Nickname</td><td>IP</td>");
		    fprintf(Glob_outfp, "<td>Hits</td>");
		    fprintf(Glob_outfp, "<td>Owner</td> </tr>\n");

		    for (i = 1; i <= Glob_maxws; i++) {
			fprintf(Glob_outfp, "<tr><td>%s</td>",\
			        Glob_wsdata[i].nick);
			fprintf(Glob_outfp, "<td>%s</td>", \
			        Glob_wsdata[i].ip);
			fprintf(Glob_outfp, "<td><center>%d</center></td>", \
				Glob_hitlist[i]);
			fprintf(Glob_outfp, "<td>%s %s</td> </tr>\n", \
				Glob_wsdata[i].first, Glob_wsdata[i].last);
		    }
		    fprintf(Glob_outfp, "</table></body></html>\n");
		    display_flywheel(DONE);
		    fclose(Glob_outfp);
		}
		break;
	    }
	}
    }
}


/* -------------------------------------------------------------
    THE ABSOLUTELY MAIN CORE FUNCTION OF THIS PROGRAM!
    This opens the Apache access_log file, scans it for number of
    relevant "hits", rewinds the file pointer if output is required
    Opens a output file if required, generates output.
    
    Input is outmode, workstation id and page id

    If outmode = ALLNOVIEW, then it is being used by the "scan_all_ws"
    function, where it just fills up the global array Glob_hitlist with
    hits for all workstations.

    If workstation id is 0 then log file lines are checked against
    all known workstations from the workstation database file. Otherwise
    the line is checked only against the id value passed.  

    Similarly, if page id is 0 then log file lines are checked against
    all known pages from the page database file. Otherwise
    the line is checked only against the id value passed.  
    
    This gives 4 conditions:
    wsid = known value, pageid = knownvalue
    wsid = known value, pageid = unknownvalue
    wsid = unknown value, pageid = knownvalue
    wsid = unknown value, pageid = unknownvalue
    
    Works for all our menu options!!

   ------------------------------------------------------------- */
int scan_access_log(int outmode, int wsid, int pageid)
{
    int hits, k, adate, getout =  0;
    char line[1024];
    FILE *fp;

    if ((fp = fopen(Glob_access_log_file, "r")) == NULL)
	quit(EXIT_FILE_ERROR, Glob_access_log_file);

    if (outmode == ALLNOVIEW)
	getout = 1;
    outmode = NOVIEW;

    printf("\nScanning for above parameters in log file .... ");

    while(1) {    
	if (outmode != VIEW)
	    hits = 0;

	for (k = 0; k <= Glob_maxws; k++)
	    Glob_hitlist[k] = 0;

        while ((fgets(line, 1024, fp)) != NULL) {
	    if (asciiz_line(line) == 0)
		continue;

    	    currline_parse(line);
	    adate = logdate_to_int(Glob_currline.adate);

	    if ((adate < Glob_userstart_date) || (adate > Glob_userend_date))
		continue;

	    if (outmode != VIEW)
		display_flywheel(hits);

	    if ((k = currline_ip_match(wsid)) && \
		currline_page_match(pageid)) {
		if (output_log(outmode, wsid, pageid) == -1)
		    break;
		if (outmode != VIEW) {
		    hits++;
		    Glob_hitlist[k]++;
		}
	    }
	}	/* inner while loop */

	if (outmode == HTMLFILE) {
	    fprintf(Glob_outfp, "</table></body></html>\n");
	    fclose(Glob_outfp);
	}
	else if (outmode == TXTFILE)
	    fclose(Glob_outfp);

	if (outmode != VIEW)
	    display_flywheel(DONE);

	if (getout) {
    	    fclose(fp);
	    return (hits);
        }

	while(1) {
	    if ((outmode = output_format_menu()) == NOVIEW) {
		fclose(fp);
		return(hits);
	    }
	    if (output_header(outmode, hits, wsid, pageid) == 0)
		break;
	}
	rewind(fp);
    }
}


/* -------------------------------------------------------------
    Used to parse the log file line into a global structure,
    currline, so it can be checked, printed, saved, displayed, etc.
    if the conditions match.

    An example of the Apache log file format is:

    33.214.240.22 - - [10/Oct/2000:12:53:47 +0530]   --> note line break
     "GET /gtlrnd/images/ HTTP/1.0" 200 13168	     --> These 2 are one line

    In order this is:

    IP
    Date
    Method
    Page name
    Version
    Error code
    File size
   ------------------------------------------------------------- */
void currline_parse(char *line)
{
    strcpy(Glob_currline.ip, strtok(line, " "));
    strtok(NULL, "[");
    strcpy(Glob_currline.adate, strtok(NULL, "]"));
    strtok(NULL, "\"");
    strcpy(Glob_currline.amethod, strtok(NULL, " "));
    strcpy(Glob_currline.page, strtok(NULL, " "));
    strcpy(Glob_currline.version, strtok(NULL, "\""));
    strcpy(Glob_currline.aerror, strtok(NULL, " "));
    strcpy(Glob_currline.fsize, strtok(NULL, " \n"));
}


/* -------------------------------------------------------------
    Displays a flywheel while reading or writing files, so that
    the user gets an indication something os going on.
    Its not required for small log files, but try without it with a
    10mb log file.
    
    If i = DONE then it prints "Done" on screen.
    Otherwise uses the "mod" operator to print "-\|/" in succesion
   ------------------------------------------------------------- */
void display_flywheel(int i)
{
    if (i == DONE) {
        printf("\b Done\n");
	return;
    }
    
    switch(i % 4)
    {
	case 0:
	    printf("\b-");
	    break;
	case 1:
	    printf("\b\\");
	    break;
	case 2:
	    printf("\b|");
	    break;
	case 3:
	    printf("\b/");
	    break;
    }
}

/* -------------------------------------------------------------
    Returns the index number of the workstation data array if the
    current log file line has a match.
    
    If 0 is the parameter, then all KNOWN IP values are checked
    against the line's IP value and returns the index if there is
    a match.
    
    If an index value is passed, then index is returned if the
    current line contains that IP number.

    Known IP values are those in the workstation database file.
   ------------------------------------------------------------- */
int currline_ip_match(int i)
{
    if (i) {
	if (strcmp(Glob_currline.ip, Glob_wsdata[i].ip) == 0)
	    return(i);
    }
    else {
	for (i = 1; i <= Glob_maxws; i++) {
	    if (strcmp(Glob_currline.ip, Glob_wsdata[i].ip) == 0)
		return(i);
	}
    }
    return(0);
}

/* -------------------------------------------------------------
    Returns the index number of the page data array if the
    current log file line has a match.
    
    If 0 is the parameter, then all KNOWN page names are checked
    against the line's page name and returns the index if there is
    a match.
    
    If an index value is passed, then index is returned if the
    current line contains that page name.
    
    Known page names are those in the page database file.
   ------------------------------------------------------------- */
int currline_page_match(int i)
{
    if (i) {
	if (strcmp(Glob_pagedata[i].pname, Glob_currline.page) == 0)
	    return(i);
    }
    else {
	for (i = 1; i <= Glob_maxpages; i++) {
	    if (strcmp(Glob_pagedata[i].pname, Glob_currline.page) == 0)
		return(i);
	}
    }
    return(0);
}


/* -------------------------------------------------------------
    Open a file for output-append. If a file exists, prompt an error.
    
    If a passed name is a workstation database filename or if it is
    a page database filename, then just open it for append.
   ------------------------------------------------------------- */
int get_output_filename(char *s)
{
    char first[MAXLEN];

    if (strlen(s))
	strcpy(first, s);
    else {
	if (ngets("output file name", first, MAXLEN) == 0)
	    return (-1);
	if ((Glob_outfp = fopen(first, "r")) != NULL) {
	    printf("File <%s> exists already. Enter another name.\n", first);
	    fclose(Glob_outfp);
	    prompt_enter();
	    return (-1);
	}
    }

    if ((Glob_outfp = fopen(first, "a")) == NULL) {
	printf("Error opening <%s> for writing!\n", first);
	return (-1);
    }
    return (0);
}

/* -------------------------------------------------------------
    Create a header for output data according to mode specified.
    
    Open-create a file if required.
    
    If totalhits is a negative value, then it is being used for the
    "scan_all_ws" function, so output header is slightly different.
    
   ------------------------------------------------------------- */
int output_header(int mode, int totalhits, int wsid, int pageid)
{
    char *sf = "\nSorted from date (yyyymmdd) %d to date (yyyymmdd) %d\n\n";

    disp_counter(DISP_INIT);

    if (mode == VIEW) {
	if (totalhits >= 0) {
	    clearscreen("MIS Report");
	    printf("Total hits: %d\n", totalhits);
	}
	else {
	    clearscreen("MIS of all Workstation Hits");
	    printf("Total hits: %d\n", (-1 * totalhits));
	}

	if (wsid) {
	    puts("Sorted for workstation:");
	    show_ws_details(wsid);
	}
	else
	    puts("Sorted for all database workstations");

	if (pageid) {
	    puts("\nSorted for page:");
	    show_page_details(pageid);
	}
	else
	    puts("\nSorted for all database pages");

	prompt_enter();
	printf(sf, Glob_userstart_date, Glob_userend_date);
	return (0);
    }

    if (get_output_filename("") == -1)
	return (-1);

    printf("\nWriting to file .... ");

    if (mode == TXTFILE) {
	fprintf(Glob_outfp, "Workstation Report\n\n");
	if (wsid) {
	    fprintf(Glob_outfp, "Sorted for following workstation:\n\n");
	    fprintf(Glob_outfp, "\tNickname: %s  IP: %s\n", \
		    Glob_wsdata[wsid].nick, Glob_wsdata[wsid].ip);
	    fprintf(Glob_outfp, "\tName: %s %s\n", \
		    Glob_wsdata[wsid].first, Glob_wsdata[wsid].last);
	    fprintf(Glob_outfp, "\tPhone: %s\n\n", Glob_wsdata[wsid].phone);
	}
	else
	    fprintf(Glob_outfp, "Sorted for all database workstations\n");

	if (pageid) {
	    fprintf(Glob_outfp, "Sorted for following page:\n\n");
	    fprintf(Glob_outfp, "\tNickname: %s\n\tPage name: %s\n", \
		    Glob_pagedata[pageid].nick, Glob_pagedata[pageid].pname);
	}
	else
	    fprintf(Glob_outfp, "Sorted for all database pages\n");

	if (totalhits >= 0)
	    fprintf(Glob_outfp, "\n\nTotal hits: %d", totalhits);
	else {
	    fprintf(Glob_outfp, "\n\nMIS of all Workstation Hits\n\n");
	    fprintf(Glob_outfp, "Total hits: %d\n", (-1 * totalhits));
	}
	fprintf(Glob_outfp, sf, Glob_userstart_date, Glob_userend_date);
    }
    else if (mode == HTMLFILE) {
	fprintf(Glob_outfp, \
		"<html><head><title>MIS Report</title></head><body>\n");
	fprintf(Glob_outfp, "<h1>Workstation Report</h1><hr>");
	if (wsid) {
	    fprintf(Glob_outfp, \
		    "<b>Sorted for following workstation:</b><br>\n");
	    fprintf(Glob_outfp, "<table border=1>\n<tr><td>");
	    fprintf(Glob_outfp, \
		    "Nickname: %s</td><td>IP: %s</td></tr>\n<tr><td>",\
		    Glob_wsdata[wsid].nick, Glob_wsdata[wsid].ip);
	    fprintf(Glob_outfp, \
		    "Name: %s %s</td><td>Phone: %s</td></tr>\n</table><br>\n",\
		    Glob_wsdata[wsid].first, Glob_wsdata[wsid].last, \
		    Glob_wsdata[wsid].phone);
	}
	else
	    fprintf(Glob_outfp, \
		    "<b>Sorted for all database workstations</b><br>\n");
	
	if (pageid) {
	    fprintf(Glob_outfp, "<b>Sorted for following page:</b><br>\n");
	    fprintf(Glob_outfp, "<table border=1>\n<tr><td>");
	    fprintf(Glob_outfp, \
    "Nickname: %s</td></tr><tr><td>Page name: %s</td></tr></table><br>\n", \
		    Glob_pagedata[pageid].nick, Glob_pagedata[pageid].pname);
	}
	else
	    fprintf(Glob_outfp, "<b>Sorted for all database pages</b><br>\n");

	if (totalhits >= -1)
	    fprintf(Glob_outfp, "<b>Total hits: %d</b><br><b>", totalhits);
	else {
	    fprintf(Glob_outfp, "<b>MIS of all Workstation Hits</b><br><b>\n");
	    fprintf(Glob_outfp, "Total hits: %d\n <br>", (-1 * totalhits));
	}
	fprintf(Glob_outfp, sf, Glob_userstart_date, Glob_userend_date);
	fprintf(Glob_outfp, "</b><p><table border=1>\n");
    }
    return(0);
}

/* -------------------------------------------------------------
    Write data to the output-mode specified file.
    This is called from the main scan_access_log function to print
    a line according to mode.
   ------------------------------------------------------------- */
int output_log(int mode, int wsid, int pageid)
{
    int i, j;

    if (mode == NOVIEW)
	return;
    else if (mode == VIEW) {
	printf("%s", Glob_currline.adate);
	if (wsid == 0)
	    printf("\t%s", Glob_currline.ip);
	if (pageid == 0)
	    printf("\t%s", Glob_currline.page);
	printf("\n");
	if ((j = disp_counter(10)) == DISP_EXIT)
	    return (-1);
	if (j == DISP_CONTINUE)
	    underline(LINE80);
    }
    else if (mode == TXTFILE) {
	fprintf(Glob_outfp, "%s", Glob_currline.adate);
	if (wsid == 0)
	    fprintf(Glob_outfp, "\t%s", Glob_currline.ip);
	if (pageid == 0)
	    fprintf(Glob_outfp, "\t%s", Glob_currline.page);
	fprintf(Glob_outfp, "\n");
    }
    else if (mode == HTMLFILE) {
	fprintf(Glob_outfp, "<tr><td>%s</td>", Glob_currline.adate);
	if (wsid == 0)
	    fprintf(Glob_outfp, "<td>%s</td>", Glob_currline.ip);
	if (pageid == 0)
	    fprintf(Glob_outfp, "<td>%s</td>", Glob_currline.page);
	fprintf(Glob_outfp, "</tr>\n");
    }
}









